searchNodes=[{"doc":"Documentation for Hammer module. This is the main API for the Hammer rate-limiter. This module assumes a backend pool has been started, most likely by the Hammer application.","ref":"Hammer.html","title":"Hammer","type":"module"},{"doc":"Check if the action you wish to perform is within the bounds of the rate-limit. Args: id : String name of the bucket. Usually the bucket name is comprised of some fixed prefix, with some dynamic string appended, such as an IP address or user id. scale_ms : Integer indicating size of bucket in milliseconds limit : Integer maximum count of actions within the bucket Returns either {:allow, count} , {:deny, limit} or {:error, reason} Example: user_id = 42076 case check_rate ( &quot;file_upload: \#{ user_id } &quot; , 60_000 , 5 ) do { :allow , _count } -&gt; # do the file upload { :deny , _limit } -&gt; # render an error page or something end","ref":"Hammer.html#check_rate/3","title":"Hammer.check_rate/3","type":"function"},{"doc":"Same as check_rate/3 , but allows specifying a backend.","ref":"Hammer.html#check_rate/4","title":"Hammer.check_rate/4","type":"function"},{"doc":"Same as check_rate/3, but allows the increment number to be specified. This is useful for limiting apis which have some idea of 'cost', where the cost of each hit can be specified.","ref":"Hammer.html#check_rate_inc/4","title":"Hammer.check_rate_inc/4","type":"function"},{"doc":"Same as check_rate_inc/4, but allows specifying a backend.","ref":"Hammer.html#check_rate_inc/5","title":"Hammer.check_rate_inc/5","type":"function"},{"doc":"Delete all buckets belonging to the provided id, including the current one. Effectively resets the rate-limit for the id. Arguments: id : String name of the bucket Returns either {:ok, count} where count is the number of buckets deleted, or {:error, reason} . Example: user_id = 2406 { :ok , _count } = delete_buckets ( &quot;file_uploads: \#{ user_id } &quot; )","ref":"Hammer.html#delete_buckets/1","title":"Hammer.delete_buckets/1","type":"function"},{"doc":"Same as delete_buckets/1, but allows specifying a backend","ref":"Hammer.html#delete_buckets/2","title":"Hammer.delete_buckets/2","type":"function"},{"doc":"Inspect bucket to get count, count_remaining, ms_to_next_bucket, created_at, updated_at. This function is free of side-effects and should be called with the same arguments you would use for check_rate if you intended to increment and check the bucket counter. Arguments: id : String name of the bucket. Usually the bucket name is comprised of some fixed prefix,with some dynamic string appended, such as an IP address or user id. scale_ms : Integer indicating size of bucket in milliseconds limit : Integer maximum count of actions within the bucket Returns either {:ok, {count, count_remaining, ms_to_next_bucket, created_at, updated_at} , or {:error, reason} . Example: inspect_bucket ( &quot;file_upload:2042&quot; , 60_000 , 5 ) { :ok , { 1 , 2499 , 29381612 , 1450281014468 , 1450281014468 } }","ref":"Hammer.html#inspect_bucket/3","title":"Hammer.inspect_bucket/3","type":"function"},{"doc":"Same as inspect_bucket/3, but allows specifying a backend","ref":"Hammer.html#inspect_bucket/4","title":"Hammer.inspect_bucket/4","type":"function"},{"doc":"Make a rate-checker function, with the given id prefix, scale_ms and limit. Arguments: id_prefix : String prefix to the id scale_ms : Integer indicating size of bucket in milliseconds limit : Integer maximum count of actions within the bucket Returns a function which accepts an id suffix, which will be combined with the id_prefix . Calling this returned function is equivalent to: Hammer.check_rate(&quot;\#{id_prefix}\#{id}&quot;, scale_ms, limit) Example: chat_rate_limiter = make_rate_checker ( &quot;send_chat_message:&quot; , 60_000 , 20 ) user_id = 203517 case chat_rate_limiter . ( user_id ) do { :allow , _count } -&gt; # allow chat message { :deny , _limit } -&gt; # deny end","ref":"Hammer.html#make_rate_checker/3","title":"Hammer.make_rate_checker/3","type":"function"},{"doc":"","ref":"Hammer.html#make_rate_checker/4","title":"Hammer.make_rate_checker/4","type":"function"},{"doc":"Hammer application, responsible for starting the backend worker pools. Configured with the :hammer environment key: :backend , Either a tuple of {module, config} , or a keyword-list of separate, named backends. Examples: {Hammer.Backend.ETS, []} , [ets: {Hammer.Backend.ETS, []}, ...] :suppress_logs , if set to true , stops all log messages from Hammer General Backend Options Different backends take different options, but all will accept the following options, and with the same effect: :expiry_ms (int): expiry time in milliseconds, after which a bucket will be deleted. The exact mechanism for cleanup will vary by backend. This configuration option is mandatory :pool_size (int): size of the backend worker pool (default=2) :pool_max_overflow int(): number of extra workers the pool is permitted to spawn when under pressure. The worker pool (managed by the poolboy library) will automatically create and destroy workers up to the max-overflow limit (default=0) Example of a single backend: config :hammer , backend : { Hammer.Backend.ETS , [ expiry_ms : 60_000 * 60 * 2 ] } Example of config for multiple-backends: config :hammer , backend : [ ets : { Hammer.Backend.ETS , [ ets_table_name : :hammer_backend_ets_buckets , expiry_ms : 60_000 * 60 * 2 , cleanup_interval_ms : 60_000 * 2 , ] } , redis : { Hammer.Backend.Redis , [ expiry_ms : 60_000 * 60 * 2 , redix_config : [ host : &quot;localhost&quot; , port : 6379 ] , pool_size : 4 , ] } ]","ref":"Hammer.Application.html","title":"Hammer.Application","type":"module"},{"doc":"Callback implementation for Application.start/2 .","ref":"Hammer.Application.html#start/2","title":"Hammer.Application.start/2","type":"function"},{"doc":"The backend Behaviour module.","ref":"Hammer.Backend.html","title":"Hammer.Backend","type":"behaviour"},{"doc":"","ref":"Hammer.Backend.html#c:count_hit/3","title":"Hammer.Backend.count_hit/3","type":"callback"},{"doc":"","ref":"Hammer.Backend.html#c:count_hit/4","title":"Hammer.Backend.count_hit/4","type":"callback"},{"doc":"","ref":"Hammer.Backend.html#c:delete_buckets/2","title":"Hammer.Backend.delete_buckets/2","type":"callback"},{"doc":"","ref":"Hammer.Backend.html#c:get_bucket/2","title":"Hammer.Backend.get_bucket/2","type":"callback"},{"doc":"","ref":"Hammer.Backend.html#t:bucket_info/0","title":"Hammer.Backend.bucket_info/0","type":"type"},{"doc":"","ref":"Hammer.Backend.html#t:bucket_key/0","title":"Hammer.Backend.bucket_key/0","type":"type"},{"doc":"An ETS backend for Hammer. The public API of this module is used by Hammer to store information about rate-limit 'buckets'. A bucket is identified by a key , which is a tuple {bucket_number, id} . The essential schema of a bucket is: {key, count, created_at, updated_at} , although backends are free to store and retrieve this data in whichever way they wish. Use start or start_link to start the server: { :ok , pid } = Hammer.Backend.ETS . start_link ( args ) args is a keyword list: expiry_ms : (integer) time in ms before a bucket is auto-deleted, should be larger than the expected largest size/duration of a bucket cleanup_interval_ms : (integer) time between cleanup runs, Example: Hammer.Backend.ETS . start_link ( expiry_ms : 1000 * 60 * 60 , cleanup_interval_ms : 1000 * 60 * 10 )","ref":"Hammer.Backend.ETS.html","title":"Hammer.Backend.ETS","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Hammer.Backend.ETS.html#child_spec/1","title":"Hammer.Backend.ETS.child_spec/1","type":"function"},{"doc":"Record a hit in the bucket identified by key","ref":"Hammer.Backend.ETS.html#count_hit/3","title":"Hammer.Backend.ETS.count_hit/3","type":"function"},{"doc":"Record a hit in the bucket identified by key , with a custom increment","ref":"Hammer.Backend.ETS.html#count_hit/4","title":"Hammer.Backend.ETS.count_hit/4","type":"function"},{"doc":"Delete all buckets associated with id .","ref":"Hammer.Backend.ETS.html#delete_buckets/2","title":"Hammer.Backend.ETS.delete_buckets/2","type":"function"},{"doc":"Retrieve information about the bucket identified by key","ref":"Hammer.Backend.ETS.html#get_bucket/2","title":"Hammer.Backend.ETS.get_bucket/2","type":"function"},{"doc":"Callback implementation for GenServer.init/1 .","ref":"Hammer.Backend.ETS.html#init/1","title":"Hammer.Backend.ETS.init/1","type":"function"},{"doc":"","ref":"Hammer.Backend.ETS.html#start/0","title":"Hammer.Backend.ETS.start/0","type":"function"},{"doc":"","ref":"Hammer.Backend.ETS.html#start/1","title":"Hammer.Backend.ETS.start/1","type":"function"},{"doc":"","ref":"Hammer.Backend.ETS.html#start_link/0","title":"Hammer.Backend.ETS.start_link/0","type":"function"},{"doc":"","ref":"Hammer.Backend.ETS.html#start_link/1","title":"Hammer.Backend.ETS.start_link/1","type":"function"},{"doc":"","ref":"Hammer.Backend.ETS.html#stop/0","title":"Hammer.Backend.ETS.stop/0","type":"function"},{"doc":"","ref":"Hammer.Backend.ETS.html#t:bucket_info/0","title":"Hammer.Backend.ETS.bucket_info/0","type":"type"},{"doc":"","ref":"Hammer.Backend.ETS.html#t:bucket_key/0","title":"Hammer.Backend.ETS.bucket_key/0","type":"type"},{"doc":"Top-level Supervisor for the Hammer application. Starts a set of poolboy pools based on provided configuration, which are latter called to by the Hammer module. See the Application module for configuration examples.","ref":"Hammer.Supervisor.html","title":"Hammer.Supervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Hammer.Supervisor.html#child_spec/1","title":"Hammer.Supervisor.child_spec/1","type":"function"},{"doc":"Callback implementation for Supervisor.init/1 .","ref":"Hammer.Supervisor.html#init/1","title":"Hammer.Supervisor.init/1","type":"function"},{"doc":"","ref":"Hammer.Supervisor.html#start_link/2","title":"Hammer.Supervisor.start_link/2","type":"function"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Changed Updgrade dependency packages Merged https://github.com/ExHammer/hammer/pull/41 resulting in ETC without GenServer (and therefore better performance) Merged https://github.com/ExHammer/hammer/pull/46 remove additional whitespace Updated Docs based on https://github.com/ExHammer/hammer/pull/45 Adds CREDITS.md","ref":"changelog.html#6-1-0-2022-06-13","title":"Changelog - 6.1.0 - 2022-06-13","type":"extras"},{"doc":"Changed Change the ETS backend to throw an error if either expiry_ms or cleanup_interval_ms config values are missing. This should have been fixed ages ago. Default :pool_max_overflow changed to 0 . It's a better default, given that some users have seen weird errors when using a higher overflow. In general, capacity should be increased by using a higher :pool_size instead Changed how the ETS backend does cleanups of data, should be more performant.","ref":"changelog.html#6-0-0-2018-10-12","title":"Changelog - 6.0.0 - 2018-10-12","type":"extras"},{"doc":"Added A new check_rate_inc function, which allows the caller to specify the integer with which to increment the bucket by. This is useful for limiting APIs which have some notion of &quot;cost&quot; per call.","ref":"changelog.html#5-0-0-2018-05-18","title":"Changelog - 5.0.0 - 2018-05-18","type":"extras"},{"doc":"Changed Use a worker-pool for the backend (via poolboy), this avoids bottle-necking all traffic through a single hammer process, thus improving throughput for the system overall Added New configuration options for backends: :pool_size , determines the number of workers in the pool (default 4) :pool_max_overflow , maximum extra workers to be spawned when the system is under pressure (default 4) Multiple instances of the same backend! You can now have two ETS backends, fifteen Redis's, whatever you want","ref":"changelog.html#4-0-0-2018-04-23","title":"Changelog - 4.0.0 - 2018-04-23","type":"extras"},{"doc":"Changed Require elixir &gt;= 1.6 Use a more sane supervision tree structure","ref":"changelog.html#3-0-0-2018-02-20","title":"Changelog - 3.0.0 - 2018-02-20","type":"extras"},{"doc":"Changed Add option to use more than one backend Add option to suppress all logging","ref":"changelog.html#2-1-0-2017-11-25","title":"Changelog - 2.1.0 2017-11-25","type":"extras"},{"doc":"Changed New, simpler API No longer need to start backend processes manually Call Hammer.check_rate directly, rather than use ing a macro Hammer is now an OTP application, configured via Mix.Config","ref":"changelog.html#2-0-0-2017-09-24","title":"Changelog - 2.0.0 - 2017-09-24","type":"extras"},{"doc":"Added Formalise backend API in Hammer.Backend behaviour","ref":"changelog.html#1-0-0-2017-08-22","title":"Changelog - 1.0.0 - 2017-08-22","type":"extras"},{"doc":"Changed Minor fixes","ref":"changelog.html#0-2-1-2017-08-10","title":"Changelog - 0.2.1 - 2017-08-10","type":"extras"},{"doc":"Hammer A rate-limiter for Elixir, with pluggable storage backends.","ref":"readme.html","title":"Readme","type":"extras"},{"doc":"We've just released a new helper-library to make adding rate-limiting to your Phoenix (or other plug-based) application even easier: Hammer.Plug .","ref":"readme.html#new-hammer-plug","title":"Readme - New! Hammer-Plug","type":"extras"},{"doc":"Hammer is available in Hex , the package can be installed by adding :hammer to your list of dependencies in mix.exs : def deps do [ { :hammer , &quot;~&gt; 6.1&quot; } ] end","ref":"readme.html#installation","title":"Readme - Installation","type":"extras"},{"doc":"On HexDocs: https://hexdocs.pm/hammer/frontpage.html The Tutorial is an especially good place to start.","ref":"readme.html#documentation","title":"Readme - Documentation","type":"extras"},{"doc":"Example: defmodule MyApp.VideoUpload do def upload ( video_data , user_id ) do case Hammer . check_rate ( &quot;upload_video: \#{ user_id } &quot; , 60_000 , 5 ) do { :allow , _count } -&gt; # upload the video, somehow { :deny , _limit } -&gt; # deny the request end end end The Hammer module provides the following functions: check_rate(id, scale_ms, limit) check_rate_inc(id, scale_ms, limit, increment) inspect_bucket(id, scale_ms, limit) delete_buckets(id) Backends are configured via Mix.Config : config :hammer , backend : { Hammer.Backend.ETS , [ expiry_ms : 60_000 * 60 * 4 , cleanup_interval_ms : 60_000 * 10 ] } See the Tutorial for more. See the Hammer Testbed app for an example of using Hammer in a Phoenix application.","ref":"readme.html#usage","title":"Readme - Usage","type":"extras"},{"doc":"Hammer.Backend.ETS (provided with Hammer for testing and dev purposes, not very good for production use) Hammer.Backend.Redis Hammer.Backend.Mnesia (beta)","ref":"readme.html#available-backends","title":"Readme - Available Backends","type":"extras"},{"doc":"If you're having trouble, either open an issue on this repo, or reach out to the maintainers ( @shanekilkelly ) on Twitter.","ref":"readme.html#getting-help","title":"Readme - Getting Help","type":"extras"},{"doc":"Hammer was inspired by the ExRated library, by grempe .","ref":"readme.html#acknowledgements","title":"Readme - Acknowledgements","type":"extras"},{"doc":"Copyright (c) 2017 Shane Kilkelly This library is MIT licensed. See the LICENSE for details.","ref":"readme.html#license","title":"Readme - License","type":"extras"},{"doc":"Hammer is a rate-limiter for the Elixir language. It's killer feature is a pluggable backend system, allowing you to use whichever storage suits your needs. Currently, backends for ETS, Redis , and Mnesia are available. case Hammer . check_rate ( &quot;file_upload: \#{ user_id } &quot; , 60_000 , 10 ) do { :allow , _count } -&gt; Upload . file ( data ) { :deny , _limit } -&gt; render_error_page ( ) end To get started with Hammer, read the Tutorial . See the Hammer.Application module for full documentation of configuration options. A primary goal of the Hammer project is to make it easy to implement new storage backends. See the documentation on creating backends for more details.","ref":"frontpage.html","title":"Overview","type":"extras"},{"doc":"We've just released a new helper-library to make adding rate-limiting to your Phoenix (or other plug-based) application even easier: Hammer.Plug .","ref":"frontpage.html#new-hammer-plug","title":"Overview - New! Hammer-Plug","type":"extras"},{"doc":"","ref":"tutorial.html","title":"Tutorial","type":"extras"},{"doc":"Add Hammer as a dependency in mix.exs : def deps do [ { :hammer , &quot;~&gt; 6.0&quot; } ] end","ref":"tutorial.html#installation","title":"Tutorial - Installation","type":"extras"},{"doc":"When we want to rate-limit some action, we want to ensure that the number of actions permitted is limited within a specified time-period. For example, a maximum of five times within on minute. Usually the limit is enforced per-user, per-client, or per some other unique-ish value, such as IP address. It's much rarer, but not unheard-of, to limit the action globally without taking the identity of the user or client into account. In the Hammer API, the maximum number of actions is the limit , and the timespan (in milliseconds) is the scale_ms . The combination of the name of the action with some unique identifier is the id . Hammer uses a Token Bucket algorithm to count the number of actions occurring in a &quot;bucket&quot;. If the count within the bucket is lower than the limit, then the action is allowed, otherwise it is denied.","ref":"tutorial.html#core-concepts","title":"Tutorial - Core Concepts","type":"extras"},{"doc":"To use Hammer, you need to do two things: Configure the :hammer application Use the functions in the Hammer module In this example, we will use the ETS backend, which stores data in an in-memory ETS table.","ref":"tutorial.html#usage","title":"Tutorial - Usage","type":"extras"},{"doc":"The Hammer OTP application is configured the usual way, using Mix.Config . Your project probably has a config/config.exs file, in which you should configure Hammer, like so: config :hammer , backend : { Hammer.Backend.ETS , [ expiry_ms : 60_000 * 60 * 4 , cleanup_interval_ms : 60_000 * 10 ] } The only configuration key (so far) is :backend , and its value is a tuple/pair of the backend module name, and a backend-specific keyword list of configuration options. Because expiry of stale buckets is so essential to the smooth operation of a rate-limiter, all backends will accept an :expiry_ms option, and many will also accept :cleanup_interval_ms , depending on how expiry is implemented internally. (For example, Redis supports native data expiry, and so doesn't require :cleanup_interval_ms .) The :expiry_ms value should be configured to be longer than the life of the longest bucket you will be using, as otherwise the bucket could be deleted while it is still counting up hits for its time period. The size of the backend worker pool can be tweaked with the :pool_size and :pool_max_overflow options, (which are then supplied to poolboy ). :pool_size determines the size of the pool, and :pool_max_overflow determines how many extra workers can be spawned when the system is under pressure. The default for both is 0 , which will be fine for most systems. (Note: we've seen some weird errors sometimes when using a :pool_max_overflow higher than zero. Always check how this works for you in production, and consider setting a higher :pool_size instead). Luckily, even if you don't configure :hammer at all, the application will default to the ETS backend anyway, with some sensible defaults.","ref":"tutorial.html#configuring-hammer","title":"Tutorial - Configuring Hammer","type":"extras"},{"doc":"Once the Hammer application is running (and it should just start automatically when your system starts), All you need to do is use the various functions in the Hammer module: check_rate(id::string, scale_ms::integer, limit::integer) check_rate_inc(id::string, scale_ms::integer, limit::integer, increment::integer) inspect_bucket(id::string, scale_ms::integer, limit::integer) delete_buckets(id::string) make_rate_checker(id_prefix, scale_ms, limit) The most interesting is check_rate , which checks if the rate-limit for the given id has been exceeded in the specified time- scale . Ideally, the id should be a combination of some action-specific, descriptive prefix with some data which uniquely identifies the user or client performing the action. Example: # limit file uploads to 10 per minute per user user_id = get_user_id_somehow ( ) case Hammer . check_rate ( &quot;upload_file: \#{ user_id } &quot; , 60_000 , 10 ) do { :allow , _count } -&gt; # upload the file { :deny , _limit } -&gt; # deny the request end","ref":"tutorial.html#the-hammer-module","title":"Tutorial - The Hammer Module","type":"extras"},{"doc":"The Hammer module also includes a check_rate_inc function, which allows you to specify the number by which to increment the current bucket. This is useful for rate-limiting APIs which have some idea of &quot;cost&quot;, where the cost of a given operation can be determined and expressed as an integer. Example: # Bulk file upload user_id = get_user_id_somehow ( ) n = get_number_of_files ( ) case Hammer . check_rate_inc ( &quot;upload_file_bulk: \#{ user_id } &quot; , 60_000 , 10 , n ) do { :allow , _count } -&gt; # upload all of the files { :deny , _limit } -&gt; # deny the request end","ref":"tutorial.html#custom-increments","title":"Tutorial - Custom increments","type":"extras"},{"doc":"There may come a time when ETS just doesn't cut it, for example if we end up load-balancing across many nodes and want to keep our rate-limiter state in one central store. Redis is ideal for this use-case, and fortunately Hammer supports a Redis backend . To change our application to use the Redis backend, we only need to install the redis backend package, and change the :backend tuple that is used to configure the :hammer application: # config :hammer, # backend: {Hammer.Backend.ETS, []} config :hammer , backend : { Hammer.Backend.Redis , [ expiry_ms : 60_000 * 60 * 2 , redix_config : [ host : &quot;localhost&quot; , port : 6379 ] , pool_size : 4 , pool_max_overflow : 2 ] } Then it should all Just Work™.","ref":"tutorial.html#switching-to-redis","title":"Tutorial - Switching to Redis","type":"extras"},{"doc":"Hammer can be configured to start multiple backends, which can then be referred to separately when checking a rate-limit. In this example we configure both and ETS backend under the key :in_memory , and a Redis backend under the key :redis ... config :hammer , backend : [ in_memory : { Hammer.Backend.ETS , [ expiry_ms : 60_000 * 60 * 2 ] } , redis : { Hammer.Backend.Redis , [ expiry_ms : 60_000 * 60 * 2 , redix_config : [ host : &quot;localhost&quot; , port : 6379 ] ] } ] We can then refer to these backends separately: Hammer . check_rate ( :in_memory , &quot;upload: \#{ user_id } &quot; , 60_000 , 5 ) Hammer . check_rate ( :redis , &quot;upload: \#{ user_id } &quot; , 60_000 , 5 ) When using multiple backends the backend specifier key is mandatory, there is no notion of a default backend. In version 4.0 and up, it is even possible to have multiple instances of the same backend type, like so: config :hammer , backend : [ redis_one : { Hammer.Backend.Redis , [ expiry_ms : 60_000 * 60 * 2 , redix_config : [ host : &quot;localhost&quot; , port : 6666 ] ] } redis_two : { Hammer.Backend.Redis , [ expiry_ms : 60_000 * 60 * 5 , redix_config : [ host : &quot;localhost&quot; , port : 7777 ] ] } ]","ref":"tutorial.html#advanced-using-multiple-backends-at-the-same-time","title":"Tutorial - (Advanced) using multiple backends at the same time","type":"extras"},{"doc":"See the docs for the Hammer module for full documentation on all the functions created by use Hammer . See the Hammer.Application for all configuration options. Also, consult the documentation for the backend you are using, for any extra configuration options that may be relevant. See the Creating Backends for information on creating new backends for Hammer.","ref":"tutorial.html#further-reading","title":"Tutorial - Further Reading","type":"extras"},{"doc":"See Hammer.Backend.ETS for a realistic example of a Hammer Backend module. Backends for Hammer are expected to implement the Hammer.Backend behaviour, and be named something like Hammer.Backend.Foo . Several instances of the backend worker will be started as a worker pool (using the poolboy library), and so backend processes should not assume that there is only one instance of this server process.","ref":"creatingbackends.html","title":"Creating Backends","type":"extras"},{"doc":"The expected backend api is as follows: start_link(args) args : Keyword list of configuration. The expiry_ms key is considered essential, as the backend process should delete expired buckets somehow. Depending on how the backend implements cleanup, the cleanup_interval_ms key may be required. Other keys are for the backend developer to choose, but for example, if the backend were using a database called FooDB to store buckets, then an appropriate key would be foodb_config . Example: Hammer.Backend.Foo . start_link ( expiry_ms : 60_000 * 60 , cleanup_interval_ms : 60_000 * 10 , foodb_config : [ host : &quot;localhost&quot; ] ) count_hit(pid, key, timestamp) pid : pid of the backend process being called key : The key of the current bucket, in the form of a tuple {bucket::integer, id::String} . timestamp : The current timestamp (integer) This function should increment the count in the bucket by 1. Returns: Either a Tuple of {:ok, count} where count is the current count of the bucket, or {:error, reason} . count_hit(pid, key, timestamp, increment) pid : pid of the backend process being called key : The key of the current bucket, in the form of a tuple {bucket::integer, id::String} . timestamp : The current timestamp (integer) increment : The number by which to increment the bucket This function should increment the count in the bucket by the specified increment. Returns: Either a Tuple of {:ok, count} where count is the current count of the bucket, or {:error, reason} . get_bucket(pid, key) pid : pid of the backend process being called key : The key of the current bucket, in the form of a tuple {bucket::integer, id::String} . Returns: Either a tuple of {:ok, bucket} , where bucket is a tuple of {key, count, created_at, updated_at} , key is, as usual, a tuple of {bucket_number, id} , count is the count of hits in the bucket, created_at and updated_at are integer timestamps, or {:error, reason} delete_buckets(pid, id) pid : pid of the backend process being called id : rate-limit id (string) to delete This should delete all existing buckets associated with the supplied id . Returns: Either {:ok, count} , or {:error, reason}","ref":"creatingbackends.html#the-hammer-backend-behaviour","title":"Creating Backends - The Hammer.Backend behaviour","type":"extras"}]